type t

external create : unit -> t = "al_source_create"
external attach_buffer : t -> AlBuffer.t -> unit = "al_source_attach_buffer"

external play : t -> unit = "al_source_play"
external pause : t -> unit = "al_source_pause"
external stop : t -> unit = "al_source_stop"
external rewind : t -> unit = "al_source_rewind"

external i : t -> [< `source_relative
                  |  `cone_inner_angle
                  |  `cone_outer_angle
                  |  `looping
                  |  `buffer
                  |  `source_state ] -> int -> unit = "al_source_i"

external geti : t -> [< `source_relative
                     |  `cone_inner_angle
                     |  `cone_outer_angle
                     |  `looping
                     |  `buffer
                     |  `source_state
                     |  `buffers_queued
                     |  `buffers_processed ] -> int = "al_source_geti"

let get_source_state s = match geti s `source_state with
(*  | 0x0000 -> `initial (* <= there is no 0x0000 state but its reported in bytecode !?! *)*)
  | 0x1011 -> `initial
  | 0x1012 -> `playing
  | 0x1013 -> `paused
  | 0x1014 -> `stopped
  | r      -> failwith (Printf.sprintf "unknown source state (0x%04X)" r)

external f : t -> [< `pitch
                  |  `gain
                  |  `min_gain
                  |  `max_gain
                  |  `max_distance
                  |  `rolloff_factor
                  |  `cone_outer_gain
                  |  `cone_inner_angle
                  |  `cone_outer_angle
                  |  `reference_distance ] -> float -> unit = "al_source_f"

external getf : t -> [< `pitch
                     |  `gain
                     |  `min_gain
                     |  `max_gain
                     |  `max_distance
                     |  `rolloff_factor
                     |  `cone_outer_gain
                     |  `cone_inner_angle
                     |  `cone_outer_angle
                     |  `reference_distance ] -> float = "al_source_getf"

external fv : t -> [< `position
                   |  `velocity
                   |  `direction ] -> float array -> unit = "al_source_fv"

external getfv : t -> [< `position
                      |  `velocity
                      |  `direction ] -> float array = "al_source_getfv"

let from_buffer b =
  let s = create () in
    attach_buffer s b;
    s

let load_from_file fn =
  from_buffer (AlBuffer.load_from_file fn)
